"""Generated client library for batch version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.batch.v1alpha import batch_v1alpha_messages as messages


class BatchV1alpha(base_api.BaseApiClient):
  """Generated client library for service batch version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://batch.googleapis.com/'
  MTLS_BASE_URL = 'https://batch.mtls.googleapis.com/'

  _PACKAGE = 'batch'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'BatchV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new batch handle."""
    url = url or self.BASE_URL
    super(BatchV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_jobs_taskGroups_tasks = self.ProjectsLocationsJobsTaskGroupsTasksService(self)
    self.projects_locations_jobs_taskGroups = self.ProjectsLocationsJobsTaskGroupsService(self)
    self.projects_locations_jobs = self.ProjectsLocationsJobsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsJobsTaskGroupsTasksService(base_api.BaseApiService):
    """Service class for the projects_locations_jobs_taskGroups_tasks resource."""

    _NAME = 'projects_locations_jobs_taskGroups_tasks'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsLocationsJobsTaskGroupsTasksService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Return a single Task.

      Args:
        request: (BatchProjectsLocationsJobsTaskGroupsTasksGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Task) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}/taskGroups/{taskGroupsId}/tasks/{tasksId}',
        http_method='GET',
        method_id='batch.projects.locations.jobs.taskGroups.tasks.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsJobsTaskGroupsTasksGetRequest',
        response_type_name='Task',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List Tasks associated with a job.

      Args:
        request: (BatchProjectsLocationsJobsTaskGroupsTasksListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListTasksResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}/taskGroups/{taskGroupsId}/tasks',
        http_method='GET',
        method_id='batch.projects.locations.jobs.taskGroups.tasks.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/tasks',
        request_field='',
        request_type_name='BatchProjectsLocationsJobsTaskGroupsTasksListRequest',
        response_type_name='ListTasksResponse',
        supports_download=False,
    )

  class ProjectsLocationsJobsTaskGroupsService(base_api.BaseApiService):
    """Service class for the projects_locations_jobs_taskGroups resource."""

    _NAME = 'projects_locations_jobs_taskGroups'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsLocationsJobsTaskGroupsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsJobsService(base_api.BaseApiService):
    """Service class for the projects_locations_jobs resource."""

    _NAME = 'projects_locations_jobs'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsLocationsJobsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Create a Job.

      Args:
        request: (BatchProjectsLocationsJobsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs',
        http_method='POST',
        method_id='batch.projects.locations.jobs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['jobId', 'requestId'],
        relative_path='v1alpha/{+parent}/jobs',
        request_field='job',
        request_type_name='BatchProjectsLocationsJobsCreateRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Delete a Job.

      Args:
        request: (BatchProjectsLocationsJobsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
        http_method='DELETE',
        method_id='batch.projects.locations.jobs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['reason', 'requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsJobsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get a Job specified by its resource name.

      Args:
        request: (BatchProjectsLocationsJobsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Job) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs/{jobsId}',
        http_method='GET',
        method_id='batch.projects.locations.jobs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsJobsGetRequest',
        response_type_name='Job',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List all Jobs for a project within a region.

      Args:
        request: (BatchProjectsLocationsJobsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListJobsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/jobs',
        http_method='GET',
        method_id='batch.projects.locations.jobs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/jobs',
        request_field='',
        request_type_name='BatchProjectsLocationsJobsListRequest',
        response_type_name='ListJobsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (BatchProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='batch.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='BatchProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (BatchProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='batch.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (BatchProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='batch.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (BatchProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='batch.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/operations',
        request_field='',
        request_type_name='BatchProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (BatchProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='batch.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='BatchProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (BatchProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='batch.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='BatchProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(BatchV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities Cloud IoT devices API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import exceptions


class NoFieldsSpecifiedError(exceptions.Error):
  """Error when no fields were specified for a Patch operation."""


def GetClientInstance(no_http=False):
  return apis.GetClientInstance('cloudiot', 'v1', no_http=no_http)


def GetMessagesModule(client=None):
  client = client or GetClientInstance()
  return client.MESSAGES_MODULE


class _DeviceUpdateSetting(object):
  """Small value class holding data for updating a device."""

  def __init__(self, field_name, update_mask, value):
    self.field_name = field_name
    self.update_mask = update_mask
    self.value = value


class DevicesClient(object):
  """Client for devices service in the Cloud IoT API."""

  def __init__(self, client=None, messages=None):
    self.client = client or GetClientInstance()
    self.messages = messages or GetMessagesModule(client)
    self._service = self.client.projects_locations_registries_devices

  def Create(self, parent_ref, device_id,
             blocked=None, credentials=None, metadata=None):
    """Creates a Device.

    Args:
      parent_ref: a Resource reference to a parent
        cloudiot.projects.locations.registries resource for this device.
      device_id: str, the name of the resource to create.
      blocked: bool, whether the device to create should have connections
        blocked or not.
      credentials: list of DeviceCredential, the credentials for the device.
      metadata: MetadataValue, the metadata message for the device.

    Returns:
      Device: the created device.
    """
    credentials = credentials or []

    create_req_type = (
        self.messages.CloudiotProjectsLocationsRegistriesDevicesCreateRequest)
    create_req = create_req_type(
        parent=parent_ref.RelativeName(),
        device=self.messages.Device(
            id=device_id,
            blocked=blocked,
            credentials=credentials,
            metadata=metadata
        ))

    return self._service.Create(create_req)

  def Delete(self, device_ref):
    delete_req_type = (
        self.messages.CloudiotProjectsLocationsRegistriesDevicesDeleteRequest)
    delete_req = delete_req_type(name=device_ref.RelativeName())
    return self._service.Delete(delete_req)

  def Get(self, device_ref):
    get_req_type = (
        self.messages.CloudiotProjectsLocationsRegistriesDevicesGetRequest)
    get_req = get_req_type(name=device_ref.RelativeName())
    return self._service.Get(get_req)

  def List(self, parent_ref, device_ids=None, device_num_ids=None,
           field_mask=None, limit=None, page_size=100):
    """List Devices in a DeviceRegistry.

    Args:
      parent_ref: a Resource reference to a parent
        cloudiot.projects.locations.registries resource to list.
      device_ids: list of strings, the device IDs
      device_num_ids: a list of numerical device IDs
      field_mask: list of str, the fields (in addition to `id` and `num_id`) to
        populate in the listed Devices.
      limit: int or None, the total number of results to return.
      page_size: int, the number of entries in each batch (affects requests
        made, but not the yielded results).

    Returns:
      Generator of matching devices.
    """
    list_req_type = (
        self.messages.CloudiotProjectsLocationsRegistriesDevicesListRequest)
    field_mask = ','.join(field_mask) if field_mask else None
    list_req = list_req_type(
        parent=parent_ref.RelativeName(),
        deviceIds=device_ids or [],
        deviceNumIds=device_num_ids or [],
        fieldMask=field_mask)
    return list_pager.YieldFromList(
        self._service, list_req, batch_size=page_size, limit=limit,
        field='devices', batch_size_attribute='pageSize')

  def Patch(self, device_ref, blocked=None, credentials=None, metadata=None,
            auth_method=None, log_level=None):
    """Updates a Device.

    Any fields not specified will not be updated; at least one field must be
    specified.

    Args:
      device_ref: a Resource reference to a
        cloudiot.projects.locations.registries.devices resource.
      blocked: bool, whether the device to create should have connections
        blocked or not.
      credentials: List of DeviceCredential or None. If given, update the
        credentials for the device.
      metadata: MetadataValue, the metadata message for the device.
      auth_method: GatewayAuthMethodValueValuesEnum, auth method to update on
        a gateway device.
      log_level: LogLevelValueValuesEnum, the default logging verbosity for the
        device.

    Returns:
      Device: the updated device.

    Raises:
      NoFieldsSpecifiedError: if no fields were specified.
    """
    device = self.messages.Device()

    update_settings = [
        _DeviceUpdateSetting(
            'blocked',
            'blocked',
            blocked),
        _DeviceUpdateSetting(
            'credentials',
            'credentials',
            credentials),
        _DeviceUpdateSetting(
            'metadata',
            'metadata',
            metadata),
        _DeviceUpdateSetting(
            'gatewayConfig.gatewayAuthMethod',
            'gatewayConfig.gatewayAuthMethod',
            auth_method),
        _DeviceUpdateSetting(
            'logLevel',
            'logLevel',
            log_level),
    ]
    update_mask = []
    for update_setting in update_settings:
      if update_setting.value is not None:
        arg_utils.SetFieldInMessage(device,
                                    update_setting.field_name,
                                    update_setting.value)
        update_mask.append(update_setting.update_mask)
    if not update_mask:
      raise NoFieldsSpecifiedError('Must specify at least one field to update.')
    patch_req_type = (
        self.messages.CloudiotProjectsLocationsRegistriesDevicesPatchRequest)
    patch_req = patch_req_type(
        device=device,
        name=device_ref.RelativeName(),
        updateMask=','.join(update_mask))

    return self._service.Patch(patch_req)

  def ModifyConfig(self, device_ref, data, version=None):
    """Modify a device configuration.

    Follows the API semantics, notably those regarding the version parameter: If
    0 or None, the latest version is modified. Otherwise, this update will fail
    if the version number provided does not match the latest version on the
    server.

    Args:
      device_ref: a Resource reference to a
        cloudiot.projects.locations.registries.devices resource.
      data: str, the binary data for the configuration
      version: int or None, the version of the configuration to modify.

    Returns:
      DeviceConfig: the modified DeviceConfig for the device
    """
    request_type = getattr(self.messages,
                           'CloudiotProjectsLocationsRegistriesDevices'
                           'ModifyCloudToDeviceConfigRequest')
    modify_request_type = self.messages.ModifyCloudToDeviceConfigRequest
    request = request_type(
        name=device_ref.RelativeName(),
        modifyCloudToDeviceConfigRequest=modify_request_type(
            binaryData=data,
            versionToUpdate=version
        )
    )
    return self._service.ModifyCloudToDeviceConfig(request)


class DeviceConfigsClient(object):
  """Client for device_configs service in the Cloud IoT API."""

  def __init__(self, client=None, messages=None):
    self.client = client or GetClientInstance()
    self.messages = messages or GetMessagesModule(client)
    # Get around line length limits...
    service = self.client.projects_locations_registries_devices_configVersions
    self._service = service

  def List(self, parent_ref, num_versions=None):
    """List all device configurations available for a device.

    Up to a maximum of 10 (enforced by service). No pagination.

    Args:
      parent_ref: a Resource reference to a
        cloudiot.projects.locations.registries.devices resource.
      num_versions: int, the number of device configurations to list (max 10).

    Returns:
      List of DeviceConfig
    """
    request_type = getattr(self.messages,
                           'CloudiotProjectsLocationsRegistries'
                           'DevicesConfigVersionsListRequest')
    response = self._service.List(
        request_type(name=parent_ref.RelativeName(),
                     numVersions=num_versions))
    return response.deviceConfigs


class DeviceStatesClient(object):
  """Client for device_states service in the Cloud IoT API."""

  def __init__(self, client=None, messages=None):
    self.client = client or GetClientInstance()
    self.messages = messages or GetMessagesModule(client)
    self._service = self.client.projects_locations_registries_devices_states

  def List(self, parent_ref, num_states=None):
    """List all device states available for a device.

    Up to a maximum of 10 (enforced by service). No pagination.

    Args:
      parent_ref: a Resource reference to a
        cloudiot.projects.locations.registries.devices resource.
      num_states: int, the number of device states to list (max 10).

    Returns:
      List of DeviceStates
    """
    request_type = getattr(self.messages,
                           'CloudiotProjectsLocationsRegistries'
                           'DevicesStatesListRequest')
    response = self._service.List(
        request_type(name=parent_ref.RelativeName(),
                     numStates=num_states))
    return response.deviceStates



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update Membership Binding information."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Update(base.UpdateCommand):
  """Update the Binding in a Membership.

  This command can fail for the following reasons:
  * The Membership specified does not exist.
  * The Binding does not exist in the Membership.
  * The caller does not have permission to access the Membership/Binding.
  * The Scope specified does not exist.
  * The caller did not specify the location (--location) if referring to
  location other than global.

  ## EXAMPLES

  To update the binding `BINDING_NAME` in global membership `MEMBERSHIP_NAME`
  in the active project:

    $ {command} BINDING_NAME --membership=MEMBERSHIP_NAME

  To update a Binding `BINDING_NAME` associated with regional membership
  `MEMBERSHIP_NAME`, provide the location LOCATION_NAME for the Membership where
  the Binding belongs along with membership name and associated
  Scope `SCOPE_NAME`.

  $ {command} BINDING_NAME --membership=MEMBERSHIP_NAME --scope=SCOPE_NAME
    --location=LOCATION_NAME

  """

  @classmethod
  def Args(cls, parser):
    resources.AddMembershipBindingResourceArg(
        parser,
        api_version=util.VERSION_MAP[cls.ReleaseTrack()],
        binding_help=('Name of the Membership Binding to be updated.'
                      'Must comply with RFC 1123 (up to 63 characters, '
                      'alphanumeric and \'-\')'))
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '--fleet',
        type=bool,
        help='Bindings for all the membership related scopes in the fleet would be updated.',
    )
    group.add_argument(
        '--scope',
        type=str,
        help='Scope related to the Binding.',
    )

  def Run(self, args):
    fleetclient = client.FleetClient(release_track=self.ReleaseTrack())
    mask = []
    for flag in ['fleet', 'scope']:
      if args.IsKnownAndSpecified(flag):
        mask.append(flag)
    return fleetclient.UpdateMembershipBinding(
        resources.MembershipBindingResourceName(args),
        scope=args.scope,
        fleet=args.fleet,
        mask=','.join(mask))


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.GA)
class UpdateGA(Update):
  pass



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""VMware Engine VPC network peering client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.vmware import util
from googlecloudsdk.api_lib.vmware.networks import NetworksClient
from googlecloudsdk.command_lib.util.apis import arg_utils


class NetworkPeeringClient(util.VmwareClientBase):
  """VMware Engine VPC network peering client."""

  def __init__(self):
    super(NetworkPeeringClient, self).__init__()
    self.service = self.client.projects_locations_global_networkPeerings
    self.networks_client = NetworksClient()

  def Get(self, resource):
    request = self.messages.VmwareengineProjectsLocationsGlobalNetworkPeeringsGetRequest(
        name=resource.RelativeName())
    response = self.service.Get(request)
    return response

  def Create(
      self,
      resource,
      vmware_engine_network_id,
      peer_network_id,
      peer_network_type,
      description=None,
      peer_project=None,
      peer_mtu=None,
      export_custom_routes=True,
      import_custom_routes=True,
      export_custom_routes_with_public_ip=True,
      import_custom_routes_with_public_ip=True,
      exchange_subnet_routes=True,
  ):
    project = resource.Parent().Name()
    if peer_project is None:
      peer_project = project

    parent = '/'.join(resource.RelativeName().split('/')[:-2])
    peering_id = resource.Name()
    # TODO(b/265135446) : pass the values as arguments into the constructor
    peering = self.messages.NetworkPeering(description=description)
    peer_network_type_enum = arg_utils.ChoiceEnumMapper(
        arg_name='peer-network-type',
        message_enum=self.messages.NetworkPeering
        .PeerNetworkTypeValueValuesEnum,
        include_filter=lambda x: 'UNSPECIFIED' not in x).GetEnumForChoice(
            arg_utils.EnumNameToChoice(peer_network_type))
    peering.peerNetworkType = peer_network_type_enum
    ven = self.networks_client.GetByID(project, vmware_engine_network_id)
    peering.vmwareEngineNetwork = ven.name
    if (
        peer_network_type_enum
        == self.messages.NetworkPeering.PeerNetworkTypeValueValuesEnum.VMWARE_ENGINE_NETWORK
    ):
      peering.peerNetwork = 'projects/{project}/locations/global/vmwareEngineNetworks/{network_id}'.format(
          project=peer_project, network_id=peer_network_id
      )
    else:
      peering.peerNetwork = (
          'projects/{project}/global/networks/{network_id}'.format(
              project=peer_project, network_id=peer_network_id
          )
      )
    if peer_mtu:
      peering.peer_mtu = peer_mtu
    peering.exportCustomRoutes = export_custom_routes
    peering.importCustomRoutes = import_custom_routes
    peering.exportCustomRoutesWithPublicIp = export_custom_routes_with_public_ip
    peering.importCustomRoutesWithPublicIp = import_custom_routes_with_public_ip
    peering.exchangeSubnetRoutes = exchange_subnet_routes
    request = self.messages.VmwareengineProjectsLocationsGlobalNetworkPeeringsCreateRequest(
        parent=parent, networkPeering=peering, networkPeeringId=peering_id
    )

    return self.service.Create(request)

  def Update(self, resource, description):
    peering = self.Get(resource)
    update_mask = []
    peering.description = description
    update_mask.append('description')
    request = self.messages.VmwareengineProjectsLocationsGlobalNetworkPeeringsPatchRequest(
        networkPeering=peering,
        name=resource.RelativeName(),
        updateMask=','.join(update_mask),
    )
    return self.service.Patch(request)

  def Delete(self, resource, delay_hours=None):
    return self.service.Delete(
        self.messages.VmwareengineProjectsLocationsGlobalNetworkPeeringsDeleteRequest(
            name=resource.RelativeName()
        )
    )

  def List(self, location_resource):
    location = location_resource.RelativeName()
    request = self.messages.VmwareengineProjectsLocationsGlobalNetworkPeeringsListRequest(
        parent=location
    )
    return list_pager.YieldFromList(
        self.service,
        request,
        batch_size_attribute='pageSize',
        field='networkPeerings')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for AI Platform Tensorboards API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.ai import util as api_util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import common_args
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import errors
from googlecloudsdk.command_lib.ai import validation as common_validation
from googlecloudsdk.command_lib.util.args import labels_util


class TensorboardsClient(object):
  """High-level client for the AI Platform Tensorboard surface."""

  def __init__(self,
               client=None,
               messages=None,
               version=constants.BETA_VERSION):
    self.client = client or apis.GetClientInstance(
        constants.AI_PLATFORM_API_NAME,
        constants.AI_PLATFORM_API_VERSION[version])
    self.messages = messages or self.client.MESSAGES_MODULE
    self._service = self.client.projects_locations_tensorboards
    self._version = version

  def Create(self, location_ref, args):
    if self._version == constants.ALPHA_VERSION:
      return self.CreateAlpha(location_ref, args)
    elif self._version == constants.GA_VERSION:
      return self.CreateGa(location_ref, args)
    else:
      return self.CreateBeta(location_ref, args)

  def CreateGa(self, location_ref, args):
    """Create a new Tensorboard."""
    kms_key_name = common_validation.GetAndValidateKmsKey(args)
    labels = labels_util.ParseCreateArgs(
        args, self.messages.GoogleCloudAiplatformV1Tensorboard.LabelsValue)
    tensorboard = self.messages.GoogleCloudAiplatformV1Tensorboard(
        displayName=args.display_name,
        description=args.description,
        labels=labels)
    if kms_key_name is not None:
      tensorboard.encryptionSpec = api_util.GetMessage(
          'EncryptionSpec', self._version)(
              kmsKeyName=kms_key_name)

    request = self.messages.AiplatformProjectsLocationsTensorboardsCreateRequest(
        parent=location_ref.RelativeName(),
        googleCloudAiplatformV1Tensorboard=tensorboard)

    return self._service.Create(request)

  def CreateBeta(self, location_ref, args):
    """Create a new Tensorboard."""
    kms_key_name = common_validation.GetAndValidateKmsKey(args)
    labels = labels_util.ParseCreateArgs(
        args, self.messages.GoogleCloudAiplatformV1beta1Tensorboard.LabelsValue)
    tensorboard = self.messages.GoogleCloudAiplatformV1beta1Tensorboard(
        displayName=args.display_name,
        description=args.description,
        labels=labels)
    if kms_key_name is not None:
      tensorboard.encryptionSpec = api_util.GetMessage(
          'EncryptionSpec', self._version)(
              kmsKeyName=kms_key_name)

    request = self.messages.AiplatformProjectsLocationsTensorboardsCreateRequest(
        parent=location_ref.RelativeName(),
        googleCloudAiplatformV1beta1Tensorboard=tensorboard)

    return self._service.Create(request)

  def CreateAlpha(self, location_ref, args):
    """Create a new Tensorboard."""
    kms_key_name = common_validation.GetAndValidateKmsKey(args)
    labels = labels_util.ParseCreateArgs(
        args,
        self.messages.GoogleCloudAiplatformV1alpha1Tensorboard.LabelsValue)
    tensorboard = self.messages.GoogleCloudAiplatformV1alpha1Tensorboard(
        displayName=args.display_name,
        description=args.description,
        labels=labels)

    if kms_key_name is not None:
      tensorboard.encryptionSpec = api_util.GetMessage(
          'EncryptionSpec', self._version)(
              kmsKeyName=kms_key_name)

    request = self.messages.AiplatformProjectsLocationsTensorboardsCreateRequest(
        parent=location_ref.RelativeName(),
        googleCloudAiplatformV1alpha1Tensorboard=tensorboard)
    return self._service.Create(request)

  def Get(self, tensorboard_ref):
    request = self.messages.AiplatformProjectsLocationsTensorboardsGetRequest(
        name=tensorboard_ref.RelativeName())
    return self._service.Get(request)

  def List(self, limit=1000, page_size=50, region_ref=None, sort_by=None):
    request = self.messages.AiplatformProjectsLocationsTensorboardsListRequest(
        parent=region_ref.RelativeName(),
        orderBy=common_args.ParseSortByArg(sort_by))
    return list_pager.YieldFromList(
        self._service,
        request,
        field='tensorboards',
        batch_size_attribute='pageSize',
        batch_size=page_size,
        limit=limit)

  def Delete(self, tensorboard_ref):
    request = self.messages.AiplatformProjectsLocationsTensorboardsDeleteRequest(
        name=tensorboard_ref.RelativeName())
    return self._service.Delete(request)

  def Patch(self, tensorboard_ref, args):
    """Update a Tensorboard."""
    if self._version == constants.ALPHA_VERSION:
      tensorboard = self.messages.GoogleCloudAiplatformV1alpha1Tensorboard()
      labels_value = self.messages.GoogleCloudAiplatformV1alpha1Tensorboard.LabelsValue
    elif self._version == constants.GA_VERSION:
      tensorboard = self.messages.GoogleCloudAiplatformV1Tensorboard()
      labels_value = self.messages.GoogleCloudAiplatformV1Tensorboard.LabelsValue
    else:
      tensorboard = self.messages.GoogleCloudAiplatformV1beta1Tensorboard()
      labels_value = self.messages.GoogleCloudAiplatformV1beta1Tensorboard.LabelsValue

    update_mask = []

    def GetLabels():
      return self.Get(tensorboard_ref).labels

    labels_update = labels_util.ProcessUpdateArgsLazy(args, labels_value,
                                                      GetLabels)

    if labels_update.needs_update:
      tensorboard.labels = labels_update.labels
      update_mask.append('labels')

    if args.display_name is not None:
      tensorboard.displayName = args.display_name
      update_mask.append('display_name')

    if args.description is not None:
      tensorboard.description = args.description
      update_mask.append('description')

    if not update_mask:
      raise errors.NoFieldsSpecifiedError('No updates requested.')

    if self._version == constants.ALPHA_VERSION:
      req = self.messages.AiplatformProjectsLocationsTensorboardsPatchRequest(
          name=tensorboard_ref.RelativeName(),
          googleCloudAiplatformV1alpha1Tensorboard=tensorboard,
          updateMask=','.join(update_mask))
    elif self._version == constants.GA_VERSION:
      req = self.messages.AiplatformProjectsLocationsTensorboardsPatchRequest(
          name=tensorboard_ref.RelativeName(),
          googleCloudAiplatformV1Tensorboard=tensorboard,
          updateMask=','.join(update_mask))
    else:
      req = self.messages.AiplatformProjectsLocationsTensorboardsPatchRequest(
          name=tensorboard_ref.RelativeName(),
          googleCloudAiplatformV1beta1Tensorboard=tensorboard,
          updateMask=','.join(update_mask))
    return self._service.Patch(req)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update command for the resource manager - Tag Values CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.resource_manager import tags
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_manager import operations
from googlecloudsdk.command_lib.resource_manager import tag_arguments as arguments
from googlecloudsdk.command_lib.resource_manager import tag_utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Update(base.Command):
  """Updates the specified TagValue resource's description.

    Updates the TagValue's description given the TagValue's
    <organization_id>/<tagkey_short_name>/<short_name>
    or the TagValue's numeric id tagValues/<id>
  """

  detailed_help = {
      'EXAMPLES':
          """
          To update a TagValue with id ``123'', run:

            $ {command} tagValues/123 --description="foobar"

          To update a TagValue named ``dev'' with the tagKey ``env''
          under organization ``456'',
          run:

            $ {command} 465/env/dev --description="foobar"
          """
  }

  @staticmethod
  def Args(parser):
    arguments.AddResourceNameArgToParser(parser)
    arguments.AddAsyncArgToParser(parser)
    arguments.AddDescriptionArgToParser(parser)

  def Run(self, args):
    service = tags.TagValuesService()
    messages = tags.TagMessages()

    if args.RESOURCE_NAME.find('tagValues/') == 0:
      tag_value = tag_utils.GetResource(
          args.RESOURCE_NAME, 'tagValues')
    else:
      tag_value = tag_utils.GetTagValueFromNamespacedName(
          args.RESOURCE_NAME)

    tag_value.description = args.description

    update_request = messages.CloudresourcemanagerTagValuesPatchRequest(
        name=tag_value.name, tagValue=tag_value, updateMask='description')
    op = service.Patch(update_request)

    if args.async_:
      return op

    return operations.WaitForOperation(
        op,
        'Waiting for TagValue [{}] to be updated'.format(tag_value.name),
        service=service)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'logging operations list' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core.resource import resource_projector


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.ALPHA)
class List(base.ListCommand):
  """List long running operations."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument(
        '--location', required=True, help='Location of the operations.')
    parser.add_argument(
        '--operation-filter',
        required=True,
        help='Filter expression that specifies the operations to return.')
    base.URI_FLAG.RemoveFromParser(parser)
    base.FILTER_FLAG.RemoveFromParser(parser)

    util.AddParentArgs(parser, 'operations to list')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Yields:
      A list of operations.
    """
    operation_name = util.CreateResourceName(
        util.GetParentFromArgs(args), 'locations', args.location)

    request = util.GetMessages().LoggingProjectsLocationsOperationsListRequest(
        name=operation_name, filter=args.operation_filter)

    result = util.GetClient().projects_locations_operations.List(request)
    self._cancellation_requested = False
    for operation in result.operations:
      yield operation
      if not self._cancellation_requested:
        serialize_op = resource_projector.MakeSerializable(operation)
        self._cancellation_requested = serialize_op.get('metadata', {}).get(
            'cancellationRequested', '')

  def Epilog(self, resources_were_displayed):
    if self._cancellation_requested:
      log.status.Print(
          'Note: Cancellation happens asynchronously. It may take up to 10 '
          "minutes for the operation's status to change to cancelled.")


List.detailed_help = {
    'DESCRIPTION':
        """
        Return a list of long running operation details in given LOCATION. The
        operations were scheduled by other gcloud commands. For example: a
        copy_log_entries operation scheduled by command: gcloud alpha logging
        operations copy BUCKET_ID DESTINATION --location=LOCATION. Note: while
        listing the operations, the request_type must be specified in filter.
        Example: --operation-filter=request_type=CopyLogEntries, Supported
        operation types are: CopyLogEntries, CreateBucket and UpdateBucket.
        Other supported filter expression are: operation_start_time,
        operation_finish_time and operation_state.
        """,
    'EXAMPLES':
        """\
        To list CopyLogEntries operations, run:

            $ {command} --location=LOCATION --operation-filter=request_type=CopyLogEntries

        To list CopyLogEntries operations that started after a specified time, run:

            $ {command} --location=LOCATION --operation-filter=request_type=CopyLogEntries AND operation_start_time>TIMESTAMP

        To list CopyLogEntries operations that finished before a specified time, run:

            $ {command} --location=LOCATION --operation-filter=request_type=CopyLogEntries AND operation_finish_time<TIMESTAMP

        To list CopyLogEntries operations that have a specified state, run:

            $ {command} --location=LOCATION --operation-filter=request_type=CopyLogEntries AND operation_state=STATE

        To list CopyLogEntries operations that don't have a specified state, run:

            $ {command} --location=LOCATION --operation-filter=request_type=CopyLogEntries AND operation_state!=STATE
        """
}



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from cloudsdk.google.protobuf import struct_pb2  # type: ignore


__protobuf__ = proto.module(
    package='google.spanner.v1',
    manifest={
        'KeyRange',
        'KeySet',
    },
)


class KeyRange(proto.Message):
    r"""KeyRange represents a range of rows in a table or index.

    A range has a start key and an end key. These keys can be open or
    closed, indicating if the range includes rows with that key.

    Keys are represented by lists, where the ith value in the list
    corresponds to the ith component of the table or index primary key.
    Individual values are encoded as described
    [here][google.spanner.v1.TypeCode].

    For example, consider the following table definition:

    ::

        CREATE TABLE UserEvents (
          UserName STRING(MAX),
          EventDate STRING(10)
        ) PRIMARY KEY(UserName, EventDate);

    The following keys name rows in this table:

    ::

        ["Bob", "2014-09-23"]
        ["Alfred", "2015-06-12"]

    Since the ``UserEvents`` table's ``PRIMARY KEY`` clause names two
    columns, each ``UserEvents`` key has two elements; the first is the
    ``UserName``, and the second is the ``EventDate``.

    Key ranges with multiple components are interpreted
    lexicographically by component using the table or index key's
    declared sort order. For example, the following range returns all
    events for user ``"Bob"`` that occurred in the year 2015:

    ::

        "start_closed": ["Bob", "2015-01-01"]
        "end_closed": ["Bob", "2015-12-31"]

    Start and end keys can omit trailing key components. This affects
    the inclusion and exclusion of rows that exactly match the provided
    key components: if the key is closed, then rows that exactly match
    the provided components are included; if the key is open, then rows
    that exactly match are not included.

    For example, the following range includes all events for ``"Bob"``
    that occurred during and after the year 2000:

    ::

        "start_closed": ["Bob", "2000-01-01"]
        "end_closed": ["Bob"]

    The next example retrieves all events for ``"Bob"``:

    ::

        "start_closed": ["Bob"]
        "end_closed": ["Bob"]

    To retrieve events before the year 2000:

    ::

        "start_closed": ["Bob"]
        "end_open": ["Bob", "2000-01-01"]

    The following range includes all rows in the table:

    ::

        "start_closed": []
        "end_closed": []

    This range returns all users whose ``UserName`` begins with any
    character from A to C:

    ::

        "start_closed": ["A"]
        "end_open": ["D"]

    This range returns all users whose ``UserName`` begins with B:

    ::

        "start_closed": ["B"]
        "end_open": ["C"]

    Key ranges honor column sort order. For example, suppose a table is
    defined as follows:

    ::

        CREATE TABLE DescendingSortedTable {
          Key INT64,
          ...
        ) PRIMARY KEY(Key DESC);

    The following range retrieves all rows with key values between 1 and
    100 inclusive:

    ::

        "start_closed": ["100"]
        "end_closed": ["1"]

    Note that 100 is passed as the start, and 1 is passed as the end,
    because ``Key`` is a descending column in the schema.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        start_closed (google.protobuf.struct_pb2.ListValue):
            If the start is closed, then the range includes all rows
            whose first ``len(start_closed)`` key columns exactly match
            ``start_closed``.

            This field is a member of `oneof`_ ``start_key_type``.
        start_open (google.protobuf.struct_pb2.ListValue):
            If the start is open, then the range excludes rows whose
            first ``len(start_open)`` key columns exactly match
            ``start_open``.

            This field is a member of `oneof`_ ``start_key_type``.
        end_closed (google.protobuf.struct_pb2.ListValue):
            If the end is closed, then the range includes all rows whose
            first ``len(end_closed)`` key columns exactly match
            ``end_closed``.

            This field is a member of `oneof`_ ``end_key_type``.
        end_open (google.protobuf.struct_pb2.ListValue):
            If the end is open, then the range excludes rows whose first
            ``len(end_open)`` key columns exactly match ``end_open``.

            This field is a member of `oneof`_ ``end_key_type``.
    """

    start_closed = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof='start_key_type',
        message=struct_pb2.ListValue,
    )
    start_open = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof='start_key_type',
        message=struct_pb2.ListValue,
    )
    end_closed = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof='end_key_type',
        message=struct_pb2.ListValue,
    )
    end_open = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof='end_key_type',
        message=struct_pb2.ListValue,
    )


class KeySet(proto.Message):
    r"""``KeySet`` defines a collection of Cloud Spanner keys and/or key
    ranges. All the keys are expected to be in the same table or index.
    The keys need not be sorted in any particular way.

    If the same key is specified multiple times in the set (for example
    if two ranges, two keys, or a key and a range overlap), Cloud
    Spanner behaves as if the key were only specified once.

    Attributes:
        keys (Sequence[google.protobuf.struct_pb2.ListValue]):
            A list of specific keys. Entries in ``keys`` should have
            exactly as many elements as there are columns in the primary
            or index key with which this ``KeySet`` is used. Individual
            key values are encoded as described
            [here][google.spanner.v1.TypeCode].
        ranges (Sequence[googlecloudsdk.generated_clients.gapic_clients.spanner_v1.types.KeyRange]):
            A list of key ranges. See
            [KeyRange][google.spanner.v1.KeyRange] for more information
            about key range specifications.
        all_ (bool):
            For convenience ``all`` can be set to ``true`` to indicate
            that this ``KeySet`` matches all keys in the table or index.
            Note that any keys specified in ``keys`` or ``ranges`` are
            only yielded once.
    """

    keys = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=struct_pb2.ListValue,
    )
    ranges = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message='KeyRange',
    )
    all_ = proto.Field(
        proto.BOOL,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
